# Jedi Knight NSIS Script
#
# JKPOCKET.NSI
#
# NSIS Script - Jedi Knight Pocket
#
# The JK Pocket Launcher allows you to run Jedi Knight from a removable drive whose letter
# changes as you move it to another computer. The program can be entirely self-contained
# on the drive and then used on any Windows computer.
#
# Most importantly, the JK Pocket Launcher handles Jedi Knight's registry values; it keeps
# them in a registry file alongside the JK Pocket executable, loads them in the registry
# before running jk.exe, removes them from the registry when jk.exe closes, and saves any
# settings changes that occured while jk.exe was running.
#
# If run on a computer with a proper Jedi Knight install, it backs up the local JK
# registry settings, then restores them once jk.exe is exited.
#
# [QM]
#
# (C) 1997 LucasArts Entertainment Co. All Rights Reserved
# ========================================================================================

;symbols

!define NAME "JKPocket"
!define FULLNAME "Jedi Knight Pocket"
!define APP "Jedi Knight"
!define VER "2.0.0.0"
!define DEFAULTEXE "jk.exe"
!define REGKEY "HKEY_LOCAL_MACHINE\Software\LucasArts Entertainment Company"
!define SUBKEYONE "\JediKnight"
!define SUBKEYTWO "\v1.0"

# Program Details
Name "${FULLNAME}"
OutFile "${NAME}.exe"
Caption "${FULLNAME}"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${FULLNAME}"
VIAddVersionKey Comments "Allows ${APP} to be run from a removable drive."
VIAddVersionKey CompanyName "None"
VIAddVersionKey LegalCopyright "None"
VIAddVersionKey FileDescription "${FULLNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${FULLNAME}"
VIAddVersionKey OriginalFilename "${NAME}.exe"

# Runtime Switches
CRCCheck On
WindowIcon Off
SilentInstall Silent
AutoCloseWindow True
RequestExecutionLevel user

# Include
!include "GetParameters.nsh"
!include "Registry.nsh"

# Program Icon
Icon "${NAME}.ico"

Var SECONDARYLAUNCH
Var SETTINGSDIRECTORY
Var PROGRAMEXECUTABLE
Var DEFAULTPARAMETERS
Var EXECSTRING

;end

# ========================================================================================

;code

Section "Main"
	# Check if already running
	System::Call 'kernel32::CreateMutexA(i 0, i 0, t "${NAME}") i .r1 ?e'
	Pop $0
	StrCmp $0 0 CheckINI
		StrCpy $SECONDARYLAUNCH "true"

CheckINI:
	# Start with the default values
	StrCpy "$SETTINGSDIRECTORY" "$EXEDIR"
	StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"
	StrCpy "$DEFAULTPARAMETERS" ""

	# Find the INI file, if there is one
	IfFileExists "$EXEDIR\${NAME}.ini" "" EndINI

	# Read SettingsDirectory from the INI file
	ReadINIStr $0 "$EXEDIR\${NAME}.ini" "${NAME}" "SettingsDirectory"

	# Ignore SettingsDirectory if blank
	StrCmp $0 "" NextValueOne
		StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\$0"

NextValueOne:
	# Read ProgramExecutable from the INI file
	ReadINIStr $0 "$EXEDIR\${NAME}.ini" "${NAME}" "ProgramExecutable"

	# Ignore ProgramExecutable if blank
	StrCmp $0 "" NextValueTwo
		StrCpy "$PROGRAMEXECUTABLE" "$0"

NextValueTwo:
	# Read DefaultParameters from the INI file
	ReadINIStr $DefaultPARAMETERS "$EXEDIR\${NAME}.ini" "${NAME}" "DefaultParameters"

EndINI:
	# Any missing unrequired INI entries will be an empty string, ignore associated errors
	ClearErrors

	IfFileExists "$EXEDIR\$PROGRAMEXECUTABLE" FoundEXE

	# Program executable not where expected
	MessageBox MB_OK|MB_ICONEXCLAMATION `$PROGRAMEXECUTABLE was not found. Please check your configuration.`
	Abort

FoundEXE:
	# Make sure the chosen executable won't cause a loop
	StrCmp $PROGRAMEXECUTABLE $EXEFILE "" CheckIfRunning
	Abort

CheckIfRunning:
	# Check if running
	StrCmp $SECONDARYLAUNCH "true" GetPassedParameters
	FindProcDLL::FindProc "$PROGRAMEXECUTABLE"
	StrCmp $R0 "1" WarnAnotherInstance
	StrCmp $PROGRAMEXECUTABLE ${DEFAULTEXE} GetPassedParameters
	FindProcDLL::FindProc "${DEFAULTEXE}"
	StrCmp $R0 "1" "" GetPassedParameters

WarnAnotherInstance:
	# Local instance is already running
	MessageBox MB_OK|MB_ICONINFORMATION `A local instance of ${APP} is already running. Please close other instances of ${APP} before launching ${FULLNAME}.`
	Abort

GetPassedParameters:
	# Start with no parameters
	StrCpy $EXECSTRING `"$EXEDIR\$PROGRAMEXECUTABLE"`

	# Get any passed parameters
	Call GetParameters
	Pop $0
	StrCmp "'$0'" "''" DefaultParametersCheck

	# Use launcher parameters
	StrCpy $EXECSTRING `$EXECSTRING $0`
	Goto SettingsDirectoryCheck

DefaultParametersCheck:
	StrCmp $DEFAULTPARAMETERS "" SettingsDirectoryCheck

	# Use default parameters
	StrCpy $EXECSTRING `$EXECSTRING $DEFAULTPARAMETERS`

SettingsDirectoryCheck:
	# Simply launch if an additional instance
	StrCmp $SECONDARYLAUNCH "true" LaunchAndExit

	# Check for the associated REG and DAT files
	IfFileExists "$SETTINGSDIRECTORY\${NAME}.reg" RegistryBackup
	IfFileExists "$SETTINGSDIRECTORY\${NAME}.dat" "" RegistryBackup

	# Make a copy of DAT file if REG file not found
	CopyFiles /SILENT `$SETTINGSDIRECTORY\${NAME}.dat` $SETTINGSDIRECTORY\${NAME}.reg

RegistryBackup:
	# Backup the registry
	${registry::KeyExists} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}-BackupBy${NAME}" $R0
	StrCmp $R0 "0" RestoreSettings
	${registry::KeyExists} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}" $R0
	StrCmp $R0 "-1" RestoreSettings
	${registry::MoveKey} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}" "${REGKEY}${SUBKEYONE}${SUBKEYTWO}-BackupBy${NAME}" $R0
	Sleep 100

RestoreSettings:
	# Load settings from REG file
	IfFileExists "$SETTINGSDIRECTORY\${NAME}.reg" "" StartProgramNow
	IfFileExists "$WINDIR\system32\reg.exe" "" RestoreTheKey9x
		nsExec::ExecToStack `"$WINDIR\system32\reg.exe" import "$SETTINGSDIRECTORY\${NAME}.reg"`
		Pop $R0
		StrCmp $R0 '0' StartProgramNow

RestoreTheKey9x:
	${registry::RestoreKey} "$SETTINGSDIRECTORY\${NAME}.reg" $R0

StartProgramNow:
	Sleep 100
	ExecWait $EXECSTRING

	# Save settings to REG file
	${registry::SaveKey} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}" "$SETTINGSDIRECTORY\${NAME}.reg" "" $R0
	Sleep 100

	# Restore local settings
	${registry::DeleteKey} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}" $R0
	${registry::KeyExists} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}-BackupBy${NAME}" $R0
	StrCmp $R0 "-1" ClearEmptyKeys
	${registry::MoveKey} "${REGKEY}${SUBKEYONE}${SUBKEYTWO}-BackupBy${NAME}" "${REGKEY}${SUBKEYONE}${SUBKEYTWO}" $R0

ClearEmptyKeys:
	${registry::DeleteKeyEmpty} "${REGKEY}${SUBKEYONE}" $R0
	${registry::DeleteKeyEmpty} "${REGKEY}" $R0
	Goto TheEnd

LaunchAndExit:
	Exec $EXECSTRING

TheEnd:
	${registry::Unload}

SectionEnd
;end
